/* istanbul ignore file */

const { isRelevantOperation, logDataChange, logReadAccess } = require('../utils/personalData')

module.exports = auditLogger => {
  /**
   * Starts the audit logging in case of a data change event or writes the log immediately for data read access.
   *
   * @param context - the context object
   * @returns {*|Promise} - promise if relevant for audit logging, otherwise undefined
   */
  return function (req) {
    if (isRelevantOperation(req.event, req.target)) {
      if (req.event === 'READ') {
        return logReadAccess(auditLogger, req)
      }

      return logDataChange(auditLogger, req, 'before commit')
    }
  }
}
