/* istanbul ignore file */

const { callAuditLogContinuation, isRelevantOperation, logDataChange } = require('../utils/personalData')

module.exports = auditLogger => {
  /**
   * In case of failed commit: finishes the audit logging in case of a data change event and rollback.
   * In case of failed on handler: starts audit logging
   *
   * @param context - the context object
   * @returns {Promise}
   */
  return function (req) {
    const phase = 'before rollback'

    if (isRelevantOperation(req.event, req.target) && req.event !== 'READ') {
      if (req._.auditLogContinuation) {
        return callAuditLogContinuation(req, phase)
      }

      return logDataChange(auditLogger, req, phase)
    }
  }
}
