'use strict'

/**
 * @abstract
 * @hideconstructor
 */
class Expression {
  /**
   * Create an instance of Expression.
   * This constructor is not intended to be called directly.  It is used in derived classes.
   * @param {Expression.ExpressionKind} kind kind of expression
   * @protected
   */
  constructor (kind) {
    /**
     * @type {Expression.ExpressionKind}
     * @private
     */
    this._kind = kind
  }

  /**
   * Return the expression kind.
   * @returns {Expression.ExpressionKind} the expression kind
   */
  getKind () {
    return this._kind
  }
}

/**
 * Defined kinds of expressions.
 * @enum {number}
 * @readonly
 */
Expression.ExpressionKind = {
  ALIAS: 0,
  BINARY: 1,
  LITERAL: 2,
  MEMBER: 3,
  METHOD: 4,
  TYPE_LITERAL: 5,
  UNARY: 6
}

module.exports = Expression
