'use strict'

const Transformation = require('./Transformation')

/**
 * Concat transformation.
 *
 * @extends Transformation
 * @hideconstructor
 */
class ConcatTransformation extends Transformation {
  constructor () {
    super(Transformation.TransformationKind.CONCAT)

    this._sequences = []
  }

  /**
   * Return the transformation sequences.
   * @returns {Array.<Transformation[]>} a non-empty list of lists of transformations
   */
  getSequences () {
    return this._sequences
  }

  /**
   * Add a transformation sequence.
   * @param {Transformation[]} sequence a non-empty list of transformations
   * @returns {ConcatTransformation} this concat transformation
   * @package
   */
  addSequence (sequence) {
    this._sequences.push(sequence)
    return this
  }
}

module.exports = ConcatTransformation
