'use strict'

const Transformation = require('./Transformation')

/**
 * Represents a skip transformation.
 * @extends Transformation
 * @hideconstructor
 */
class SkipTransformation extends Transformation {
  constructor () {
    super(Transformation.TransformationKind.SKIP)

    this._skip = null
  }

  /**
   * Returns the skip value.
   * @returns {number} the skip value
   */
  getSkip () {
    return this._skip
  }

  /**
   * Sets the skip value.
   * @param {number} skip the skip value
   * @returns {SkipTransformation} this transformation
   * @package
   */
  setSkip (skip) {
    this._skip = skip
    return this
  }
}

module.exports = SkipTransformation
