'use strict'

/**
 * Structure containing information about errors which occurred while processing a batch request
 */
class BatchErrorInfo {
  /**
   * Constructor
   * @param {string} id Id of the batched request where the error occurred
   * @param {OdataRequestInBatch} request Request object of the batched request whose processing failed
   * @param {OdataResponseInBatch} response Response object of the batched request whose processing failed
   */
  constructor (id, request, response) {
    /**
     * Id of the batched request where the error occurred
     * @type {string}
     * @private
     */
    this._id = id

    /**
     * Request object of the batched request whose processing failed
     * @type {OdataRequestInBatch}
     * @private
     */
    this._request = request

    /**
     * Response object of the batched request whose processing failed
     * @type {OdataResponseInBatch}
     * @private
     */
    this._response = response
  }

  /**
   * Returns the Id of the batched request where the error occurred.
   * @returns {string|*} the ID
   */
  getId () {
    return this._id
  }

  /**
   * Returns the request object whose processing failed.
   * @returns {OdataRequestInBatch} the request
   */
  getRequest () {
    return this._request
  }

  /**
   * Returns the response object whose processing failed.
   * @returns {OdataResponseInBatch} the response
   */
  getResponse () {
    return this._response
  }
}

module.exports = BatchErrorInfo
