'use strict'

const AbstractError = require('../../odata-commons').errors.AbstractError

/**
 * @extends AbstractError
 */
class NotFoundError extends AbstractError {
  /**
   * Creates an instance of  NotFoundError. The error message would be
   * "Resource '<resourceIdentifier>' could not be found". The param for resourceIdentifier
   * should be the name of the resource which could not be found.
   *
   * @param {string} resourceIdentifier The not found resource identifier.
   * @returns {NotFoundError} a new instance
   */
  static createForMissingResource (resourceIdentifier) {
    let msg = `Resource '${resourceIdentifier}' could not be found`
    return new NotFoundError(msg)
  }

  /**
   * Creates an instance of NotFoundError.
   * If an error message is not provided, the default message would be 'Resource not found'.
   *
   * @param {string} message The error message
   */
  constructor (message = 'Resource not found') {
    super(AbstractError.ErrorNames.NOT_FOUND, message)
  }
}

module.exports = NotFoundError
