'use strict'

/**
 * Container for content type, serializer function, and parameter-check function.
 */
class FormatDescription {
  /**
   * @param {ContentTypeInfo.ContentTypes} typeAndSubType Type and Subtype (e.g. application/json)
   * @param {Function} serializerFunction Serializer
   * @param {?Function} parameterCheckFunction the function to check the format parameters
   */
  constructor (typeAndSubType, serializerFunction, parameterCheckFunction) {
    this._typeAndSubType = typeAndSubType
    this._serializerFunction = serializerFunction
    this._parameterCheckFunction = parameterCheckFunction
  }

  /**
   * Return type and subtype (e.g. application/json).
   * @returns {ContentTypeInfo.ContentTypes} the type and the subtype
   */
  getTypeAndSubtype () {
    return this._typeAndSubType
  }

  /**
   * Returns the serializer function.
   * @returns {Function} the serializer
   */
  getSerializerFunction () {
    return this._serializerFunction
  }

  /**
   * Returns the parameter-check function
   * @returns {?Function} the parameter-check function
   */
  getParameterCheckFunction () {
    return this._parameterCheckFunction
  }
}

module.exports = FormatDescription
