'use strict'

const MetaProperties = require('../../odata-commons').format.JsonFormat.MetaProperties

/**
 * The service handler is responsible for getting the correct input data for the service
 * serializer. In this case we need the edm provider.
 */
class ServiceHandler {
  /**
   * This method is called when a service document request comes in.
   *
   * @param {OdataRequest} request the OData request
   * @param {OdataResponse} response the OData response
   * @param {Function} next The callback function with signature next(error, resultData).
   */
  static read (request, response, next) {
    const metadataETag = request.getService().getMetadataEtag()
    if (metadataETag) request.validateEtag(metadataETag)
    next(null, { [MetaProperties.ETAG]: metadataETag })
    // TODO this method may be replaced by a custom service handler, so we need a scenario test that ensures that this method is called...
  }
}

module.exports = ServiceHandler
