'use strict'

const commons = require('../../odata-commons')
const QueryOptions = commons.uri.UriInfo.QueryOptions
const ContentTypes = commons.format.ContentTypeInfo.ContentTypes
const RepresentationKinds = commons.format.RepresentationKind.Kinds
const ContentNegotiatorCommand = require('./ContentNegotiatorCommand')

/**
 * The `next` callback to be called upon finish execution.
 *
 * @callback Next
 * @param {?Error} error An error if there is one or null if not
 */

/**
 * Executes the content negotiation in case of debug mode.
 * @extends ContentNegotiatorCommand
 */
class ContentNegotiatorDebugCommand extends ContentNegotiatorCommand {
  /**
   * Executes the content negotiation in debug mode.
   * The content negotiation creates a `ResponseContract` object
   * as a result with all necessary content negotiation information.
   * The contract object is attached to the odata context instance.
   *
   * @param {Next} next The next callback to be called on finish
   */
  execute (next) {
    if (this._response.isHeadersSent()) {
      this._logger.debug('Headers already sent')
    } else {
      const queryOptions = this._request.getQueryOptions()
      let resultingContentType = null

      if (queryOptions[QueryOptions.ODATA_DEBUG] === 'json') {
        resultingContentType = ContentTypes.JSON
      }

      if (queryOptions[QueryOptions.ODATA_DEBUG] === 'html') {
        resultingContentType = ContentTypes.HTML
      }

      const formatManager = super.getFormatManager()
      const contract = super
        .getNegotiator()
        .negotiateContentType(formatManager, RepresentationKinds.DEBUG, resultingContentType)

      this._response.setContract(contract)

      if (contract.getContentTypeInfo()) {
        this._logger.debug('Debug response contract content type:', contract.getContentTypeInfo().toString())
      }
    }
    next()
  }
}

module.exports = ContentNegotiatorDebugCommand
