'use strict'

const RepresentationKind = require('../../odata-commons').format.RepresentationKind.Kinds
const Command = require('./Command')
const DeserializerFactory = require('../deserializer/DeserializerFactory')

/**
 * Parses the provided request body for debug output.
 * @extends Command
 */
class DebugDeserializingCommand extends Command {
  /**
   * Creates an instance of DebugDeserializingCommand.
   * @param {OdataRequest} request the current OData request
   * @param {LoggerFacade} logger the logger
   */
  constructor (request, logger) {
    super()
    this._request = request
    this._logger = logger
  }

  /**
   * Executes the request-body parsing.
   * @param {Function} next The next callback to be called on finish
   */
  execute (next) {
    if (
      this._request.getContract().getRepresentationKind() !== RepresentationKind.NO_CONTENT &&
      !this._request.getBody()
    ) {
      this._logger.debug('Deserializing request body...')
      const binaryParserFunction = DeserializerFactory.createBinaryDeserializer()
      binaryParserFunction(this._request, (error, body) => {
        this._request.setBody(Buffer.from(body).toString())
        next(error)
      })
    } else {
      next()
    }
  }
}

module.exports = DebugDeserializingCommand
