'use strict'

const commons = require('../../odata-commons')
const ContentTypes = commons.format.ContentTypeInfo.ContentTypes
const RepresentationKinds = commons.format.RepresentationKind.Kinds
const ContentNegotiatorCommand = require('./ContentNegotiatorCommand')

/**
 * The `next` callback to be called upon finish execution.
 * @callback Next
 * @param {?Error} error An error if there is one or null if not
 */

/**
 * Executes the content negotiation in case of an error.
 * @extends ContentNegotiatorCommand
 */
class ErrorContentNegotiatorCommand extends ContentNegotiatorCommand {
  /**
   * Executes the content negotiation in error mode. The content negotiation creates a `ResponseContract` object
   * as a result with all necessary content negotiation information. The contract object is
   * attached to the odata context instance.
   *
   * @param {Next} next The next callback to be called on finish
   */
  execute (next) {
    const formatManager = super.getFormatManager()
    const negotiator = super.getNegotiator()
    let contract = null
    try {
      this._logger.debug('Try to execute default content type negotiation...')
      contract = negotiator.negotiate(formatManager, this._request)
    } catch (error) {
      this._logger.debug('... Error content type negotiation failed')
      this._logger.debug('Execute content type negotation for', "'" + ContentTypes.JSON + "'")
      contract = negotiator.negotiateContentType(formatManager, RepresentationKinds.ERROR, ContentTypes.JSON)
    }
    this._logger.debug('Response contract contentTypeInfo:', contract.getContentTypeInfo())
    this._response.setContract(contract)
    next()
  }
}

module.exports = ErrorContentNegotiatorCommand
