'use strict'

const Command = require('./Command')

/**
 * The `next` callback to be called upon finish execution.
 *
 * @callback Next
 * @param {?Error} error An error if there is one or null if not
 */

/**
 * Executes the error serialization.
 *
 * @extends Command
 */
class ErrorSerializingCommand extends Command {
  /**
   * Creates an instance of the ErrorSerializingCommand.
   *
   * @param {OdataResponse} response the current OData response
   * @param {LoggerFacade} logger the logger
   */
  constructor (response, logger) {
    super()
    this._response = response
    this._logger = logger
  }

  /**
   * Executes the registered error serializing function bound with the contract created from the
   * error content negotiation.
   *
   * @param {Next} next The next callback to be called on finish
   * @param {Error} error The error thrown
   */
  execute (next, error) {
    if (this._response.isHeadersSent()) {
      this._logger.warning('Headers already sent')
      next()
    } else {
      const serialize = this._response.getContract().getSerializerFunction()
      serialize(error, this._response, (innerError, data) => {
        this._response.setBody(data)
        next(innerError)
      })
    }
  }
}

module.exports = ErrorSerializingCommand
