'use strict'

const Command = require('./Command')

/**
 * The `next` callback to be called inside the listener.
 * @callback Next
 * @param {?Error} error An error if there is one or null if not
 */

/**
 * The `listener` function which will be called by this command.
 * @callback Listener
 * @param {Context} context The current odata context instance
 * @param {Next} next The callback to be called
 */

/**
 * The EventListenerCommand executes the provided listener function.
 * @extends Command
 */
class EventListenerCommand extends Command {
  /**
   * Creates an instance of EventListenerCommand
   * @param {Context} context The current odata context instance
   * @param {Listener} listener The listener to execute
   * @param {string} name the name
   */
  constructor (context, listener, name) {
    super(context)
    this._listener = listener
    this._name = name
  }

  /**
   * Executes the listener. The listener must execute the provided next function.
   * @param {Next} next the callback to execute inside the listener
   * @param {?Error} error an error if one occurred; can be undefined
   */
  execute (next, error) {
    const context = this.getContext()
    if (this._listener) {
      this._listener(context.getRequest(), context.getResponse(), next, error)
      // The debug mode could have been changed by the debug listener.
      if (this._name === 'debug') {
        context.getResponse().setBuffered(
          context
            .getRequest()
            .getContract()
            .isDebug()
        )
      }
    } else {
      next()
    }
  }
}

module.exports = EventListenerCommand
