'use strict'

const Command = require('./Command')

/**
 * The `next` callback to be called upon finish execution.
 * @callback Next
 * @param {?Error} error An error if there is one or null if not
 */

/**
 * Executes the validation of the request headers.
 * @extends Command
 */
class OdataVersionValidationCommand extends Command {
  /**
   * Creates an instance of the OdataVersionValidationCommand.
   * @param {OdataRequest} request the current OData request
   * @param {RequestValidator} requestValidator The current request validator
   * @param {?string} version the supported OData version
   * @param {LoggerFacade} logger the logger
   */
  constructor (request, requestValidator, version, logger) {
    super()
    this._request = request
    this._requestValidator = requestValidator
    this._version = version
    this._logger = logger
  }

  /**
   * Executes the registered version validator.
   * @param {Next} next The next callback to be called on finish
   */
  execute (next) {
    this._requestValidator.validateVersion(this._version, this._request.getHeaders())
    next()
  }
}

module.exports = OdataVersionValidationCommand
