'use strict'

const Command = require('./Command')

/**
 * Executes the operation validation.
 * @extends Command
 */
class OperationValidationCommand extends Command {
  /**
   * Creates an instance of OperationValidationCommand.
   * @param {OdataRequest} request the current OData request
   * @param {RequestValidator} requestValidator The current request validator
   * @param {LoggerFacade} logger the logger
   */
  constructor (request, requestValidator, logger) {
    super()
    this._request = request
    this._requestValidator = requestValidator
    this._logger = logger
  }

  /**
   * Executes the operation validation.
   * @param {Next} next The next callback to be called on finish
   */
  execute (next) {
    this._requestValidator.validateOperationOnResource(this._request)

    next()
  }
}

module.exports = OperationValidationCommand
