'use strict'

const Command = require('./Command')
const HttpHeaderNames = require('../../odata-commons').http.HttpHeader.HeaderNames
const HttpHeader = require('../http/HttpHeader')

/**
 * Executes the parsing and validation of the prefer header.
 * @extends Command
 */
class ParsePreferHeaderCommand extends Command {
  /**
   * Creates an instance of ParsePreferHeaderCommand.
   * @param {OdataRequest} request the current OData request
   * @param {RequestValidator} requestValidator The current request validator
   * @param {LoggerFacade} logger the logger
   */
  constructor (request, requestValidator, logger) {
    super()
    this._request = request
    this._requestValidator = requestValidator
    this._logger = logger
  }

  /**
   * Executes the parsing and validation of the prefer header.
   * @param {Next} next The next callback to be called on finish
   */
  execute (next) {
    const preferHeader = this._request.getHeader(HttpHeaderNames.PREFER)
    if (preferHeader) {
      const preferences = HttpHeader.parsePreferHeader(preferHeader)
      this._request.setPreferences(preferences)
      this._logger.debug('Prefer header parsing result:', preferences)

      this._requestValidator.validatePreferences(
        preferences,
        this._request.getMethod(),
        this._request.getUriInfo().getPathSegments()
      )
    }

    next()
  }
}

module.exports = ParsePreferHeaderCommand
