'use strict'

const Command = require('./Command')

/**
 * The `next` callback to be called upon finish execution.
 * @callback Next
 * @param {?Error} error An error if there is one or null if not
 */

/**
 * Executes the query-option parsing.
 * @extends Command
 */
class QueryOptionsParserCommand extends Command {
  /**
   * Creates an instance of the QueryOptionsParserCommand.
   * @param {UriParser} uriParser the URI parser
   * @param {OdataRequest} request the OData request
   * @param {LoggerFacade} logger the logger
   */
  constructor (uriParser, request, logger) {
    super()
    this._uriParser = uriParser
    this._request = request
    this._logger = logger
  }

  /**
   * Execute the query-options parsing and change the UriInfo object accordingly.
   * @param {Next} next The callback to be called on finish
   */
  execute (next) {
    if (this._request.getQueryOptions()) {
      this._uriParser.parseQueryOptions(this._request.getQueryOptions(), this._request.getUriInfo())
      this._logger.debug('Query options result:', this._request.getUriInfo().getQueryOptions())
    }

    next()
  }
}

module.exports = QueryOptionsParserCommand
