'use strict'

const Command = require('./Command')
const InternalServerError = require('../errors/InternalServerError')

/**
 * Executes the sending of the response to the client with the provided data.
 * @extends Command
 */
class SendResponseCommand extends Command {
  /**
   * Creates an instance of the SendResponseCommand.
   * @param {OdataResponse} response the current OData response
   * @param {LoggerFacade} logger the logger
   */
  constructor (response, logger) {
    super()
    this._response = response
    this._logger = logger
  }

  /**
   * Sends the response.
   * @param {Next} next The next callback to be called on finish
   */
  execute (next) {
    if (this._response.isHeadersSent() && this._response.getBody()) {
      throw new InternalServerError(
        'Response data was already sent while there is still data available in response buffer'
      )
    }

    this._response.end(this._response.getBody(), undefined, next)
  }
}

module.exports = SendResponseCommand
