'use strict'

const validateThat = require('../../odata-commons').validator.ParameterValidator.validateThat

/**
 * The ErrorSerializer serializes an error to an OData error.
 * @abstract
 */
class ErrorSerializer {
  /**
   * Creates an instance of ErrorSerializer.
   * @param {Error} error The error to serialize.
   * @throws {IllegalArgumentError} if the provided error is undefined or does not have the correct structure
   */
  constructor (error) {
    validateThat('error', error).truthy()
    validateThat('error.message', error.message).typeOf('string')
    if (error.code) validateThat('error.code', error.code).typeOf('string')
    if (error.target) validateThat('error.target', error.target).typeOf('string')
    if (error.details) {
      validateThat('error.details', error.details)
        .array()
        .containsElementsOfType('object')
      for (let index = 0; index < error.details.length; index++) {
        validateThat('error.details[' + index + '].message', error.details[index].message).typeOf('string')
        if (error.details[index].code) {
          validateThat('error.details[' + index + '].code', error.details[index].code).typeOf('string')
        }
        if (error.details[index].target) {
          validateThat('error.details[' + index + '].target', error.details[index].target).typeOf('string')
        }
      }
    }

    this._error = error
  }
}

module.exports = ErrorSerializer
