const cds = require('../../../../cds')
const { SELECT } = cds.ql

const { getDeepSelect } = require('../../../services/utils/handlerUtils')
const { DRAFT_COLUMNS } = require('../../../../common/constants/draft')
const { getVirtuals, getVirtualFromTarget, postProcessVirtuals } = require('../../../../db/generic/virtual')

const _getColumns = target => {
  const columns = []
  for (const k in target.elements) {
    if (!target.elements[k].isAssociation && !target.elements[k].virtual && !DRAFT_COLUMNS.includes(k)) columns.push(k)
  }
  return columns
}

module.exports = async (req, srv) => {
  let deepSelect
  let virtuals = []
  if (req.event === 'draftActivate') {
    const where = Object.keys(req.target.keys)
      .filter(k => k !== 'IsActiveEntity')
      .reduce((w, k) => {
        w[k] = req.data[k]
        return w
      }, {})
    deepSelect = SELECT.from(req.target).columns(_getColumns(req.target)).where(where)
    virtuals = getVirtualFromTarget(req)
  } else if (req.event === 'UPDATE') {
    deepSelect = SELECT.from(req.query.UPDATE.entity, _getColumns(req.target))
    virtuals = getVirtualFromTarget(req)
  } else {
    deepSelect = getDeepSelect(req)
    virtuals = getVirtuals({ query: deepSelect, target: req.target }, srv.model)
  }
  const result = await cds.tx(req).run(deepSelect)
  postProcessVirtuals(virtuals, result)
  return result
}
