const cqn2cqn4sql = require('../../common/utils/cqn2cqn4sql')
const getColumns = require('../utils/columns')
const generateAliases = require('../utils/generateAliases')
const { getNavigationIfStruct } = require('../../common/utils/structured')

const _isDraft = req => {
  return (
    req.target &&
    ((typeof req.target.name === 'string' && req.target.name.endsWith('_drafts')) ||
      typeof req.target.name === 'object') // > union, which is (currently) only the case with draft
  )
}

const _isAsteriskCol = col => col === '*' || (col.ref && col.ref[0] === '*')

const _rewriteExpandAsterisks = (expand, entity, refs) => {
  const navigation = getNavigationIfStruct(entity, refs)
  const targetEntity = navigation._target

  expand.forEach(col => {
    if (col.ref && col.expand) {
      _rewriteExpandAsterisks(col.expand, targetEntity, col.ref)
    }
  })

  const asteriskColumnIndex = expand.findIndex(col => _isAsteriskCol(col))

  if (asteriskColumnIndex !== -1) {
    expand.splice(asteriskColumnIndex, 1)
    getColumns(targetEntity).forEach(col => {
      expand.push({ ref: [col.name] })
    })
  }
}

const _rewriteAsterisks = req => {
  if (!_isDraft(req) && req.query.SELECT && req.query.SELECT.columns) {
    const asteriskColumnIndex = req.query.SELECT.columns.findIndex(col => _isAsteriskCol(col))

    if (asteriskColumnIndex !== -1) {
      req.query.SELECT.columns.splice(asteriskColumnIndex, 1)
      getColumns(req.target).forEach(col => {
        req.query.SELECT.columns.push({ ref: [col.name] })
      })
    }

    req.query.SELECT.columns.forEach(col => {
      if (col.ref && col.ref[0] !== 'DraftAdministrativeData' && col.expand && req.target) {
        _rewriteExpandAsterisks(col.expand, req.target, col.ref)
      }
    })
  }
}

const _isLinked = req => {
  if (req.query.INSERT && req.query.INSERT.entries) {
    if (Array.isArray(req.query.INSERT.entries)) return req.data === req.query.INSERT.entries[0]
    else return req.data === req.query.INSERT.entries
  } else if (req.query.UPDATE && req.query.UPDATE.data) {
    return req.data === req.query.UPDATE.data
  }
}

const _restoreLink = req => {
  if (req.query.INSERT && req.query.INSERT.entries) {
    if (Array.isArray(req.query.INSERT.entries)) req.data = req.query.INSERT.entries[0]
    else req.data = req.query.INSERT.entries
  } else if (req.query.UPDATE && req.query.UPDATE.data) {
    req.data = req.query.UPDATE.data
  }
}

function handler(req) {
  // REVISIT: req.target._unresolved for join queries
  if (!this.model || typeof req.query === 'string' /* || !req.target || req.target._unresolved */) {
    return
  }

  const streaming = req.query._streaming
  const validationQuery = req.query._validationQuery

  _rewriteAsterisks(req)

  // for restore link to req.data
  const linked = _isLinked(req)

  // convert to sql cqn
  req.query = cqn2cqn4sql(req.query, this.model)

  // REVISIT: should not be necessary
  // restore link to req.data
  if (linked) _restoreLink(req)

  if (streaming) req.query._streaming = streaming
  if (validationQuery) req.query._validationQuery = validationQuery

  generateAliases(req.query, this.model)
}

handler._initial = true

module.exports = handler
