const { timestampToISO } = require('../data-conversion/timestamp')

function _arrayWithCount(a, count) {
  const _map = a.map
  const map = (..._) => _arrayWithCount(_map.call(a, ..._), count)
  return Object.defineProperties(a, {
    $count: { value: count, enumerable: false, configurable: true, writable: true },
    map: { value: map, enumerable: false, configurable: true, writable: true }
  })
}

function _createCountQuery(query) {
  const _query = JSON.parse(JSON.stringify(query))
  _query.SELECT.columns = [{ func: 'count', args: [{ val: 1 }], as: 'counted' }]
  delete _query.SELECT.groupBy
  delete _query.SELECT.limit
  delete _query.SELECT.orderBy // not necessary to keep that
  // Also change columns in sub queries
  if (_query.SELECT.from.SET) {
    _query.SELECT.from.SET.args.forEach(subCountQuery => {
      subCountQuery.SELECT.columns = [{ val: 1 }]
    })
  }
  return _query
}

const read = (executeSelectCQN, executeStreamCQN) => (model, dbc, query, req) => {
  if (query._streaming) {
    if (!query.SELECT || (query.SELECT && (!query.SELECT.columns || query.SELECT.columns.length !== 1))) {
      req.reject(400)
    }
    return executeStreamCQN(model, dbc, query, req.user, req.user.locale, timestampToISO(req.timestamp))
  }

  if (query.SELECT.count) {
    if (query.SELECT.limit) {
      const countQuery = _createCountQuery(query)
      return Promise.all([
        executeSelectCQN(model, dbc, countQuery, req.user, req.user.locale, timestampToISO(req.timestamp)),
        executeSelectCQN(model, dbc, query, req.user, req.user.locale, timestampToISO(req.timestamp))
      ]).then(([countResult, result]) => _arrayWithCount(result, countResult[0].counted))
    } else {
      return executeSelectCQN(model, dbc, query, req.user, req.user.locale, timestampToISO(req.timestamp)).then(
        result => _arrayWithCount(result, result.length)
      )
    }
  }

  return executeSelectCQN(model, dbc, query, req.user, req.user.locale, timestampToISO(req.timestamp))
}

module.exports = read
