const iterator = Symbol.iterator

module.exports = class InsertResult {
  constructor(req, results) {
    this.req = req
    this.results = results
  }

  /*
   * iterator for getting the keys
   */
  get [iterator]() {
    if (this.req.query.INSERT.as) {
      // dummy iterator with correct length
      return (super[iterator] = function* () {
        for (let i = 0; i < this.affectedRows; i++) yield {}
      })
    }

    const { target } = this.req
    if (!target || !target.keys) return (super[iterator] = this.results[iterator])
    const { entries, columns, rows, values } = this.req.query.INSERT
    const keys = Object.keys(target.keys)
    const [k1] = keys

    if (entries && k1 in entries[0]) {
      return (super[iterator] = function* () {
        for (const each of entries) {
          const kees = {}
          for (const k of keys) kees[k] = each[k]
          yield kees
        }
      })
    }

    if (columns) {
      const indices = {}
      for (const k of keys) {
        const i = columns.indexOf(k)
        if (i >= 0) indices[k] = i
      }

      if (rows && k1 in indices) {
        return (super[iterator] = function* () {
          for (const each of rows) {
            const kees = {}
            for (const k of keys) kees[k] = each[indices[k]]
            yield kees
          }
        })
      }

      if (values && k1 in indices) {
        return (super[iterator] = function* () {
          for (const each of [values]) {
            const kees = {}
            for (const k of keys) kees[k] = each[indices[k]]
            yield kees
          }
        })
      }
    }

    return (super[iterator] = function* () {
      // REVISIT: sqlite only returns a single lastID per row -> how is that with others?
      // only up to # of root entries
      const roots = this.results.slice(0, this.affectedRows)
      for (const each of roots) yield { [k1]: each.lastID || each }
    })
  }

  /*
   * the number of inserted (root) entries or the number of affectedRows in case of INSERT into SELECT
   */
  get affectedRows() {
    if (this.req.query.INSERT.as) return (super.affectedRows = this.results[0].affectedRows)
    const { entries, rows } = this.req.query.INSERT
    return (super.length = (entries && entries.length) || (rows && rows.length) || this.results.length || 1)
  }

  /*
   * for checks such as res > 2
   */
  valueOf() {
    return this.affectedRows
  }
}
