const { ensureUnlocalized } = require('../../common/utils/draft')

const ALIAS_PREFIX = 'ALIAS_'

const cleanUpName = name => {
  return ensureUnlocalized(name).replace(/\./g, '_')
}

const _redirectXpr = (xpr, definitions, aliasMap) => {
  xpr &&
    xpr.forEach(element => {
      if (element.ref) {
        if (element.ref.length > 1) {
          const view = cleanUpName(element.ref[0])
          if (aliasMap.has(view)) {
            element.ref[0] = aliasMap.get(view)
          }
        }
      } else if (element.func) {
        _redirectXpr(element.args, definitions, aliasMap)
      } else {
        _internalGenerateAliaes(element, definitions, aliasMap)
      }
    })
}

const _redirectRef = (ref, aliasMap) => {
  if (ref.as) {
    aliasMap.set(cleanUpName(ref.ref[0]), ref.as)
  } else {
    ref.as = `${ALIAS_PREFIX}${aliasMap.size + 1}`
    aliasMap.set(cleanUpName(ref.ref[0].id || ref.ref[0]), ref.as)
  }
}

const _internalGenerateAliaes = (partialCqn, definitions, aliasMap = new Map()) => {
  if (partialCqn.SELECT) {
    const selectMap = new Map(aliasMap)
    _internalGenerateAliaes(partialCqn.SELECT, definitions, selectMap)

    _redirectXpr(partialCqn.SELECT.where, definitions, selectMap)
    _redirectXpr(partialCqn.SELECT.having, definitions, selectMap)
    _redirectXpr(partialCqn.SELECT.columns, definitions, selectMap)
  } else if (partialCqn.from) {
    if (partialCqn.from.ref) {
      _redirectRef(partialCqn.from, aliasMap)
    } else {
      _internalGenerateAliaes(partialCqn.from, definitions, aliasMap)
    }
  } else if (partialCqn.join) {
    partialCqn.args.forEach(arg => {
      if (arg.ref) {
        _redirectRef(arg, aliasMap)
      } else {
        _internalGenerateAliaes(arg, definitions, aliasMap)
      }
    })
    _redirectXpr(partialCqn.on, definitions, aliasMap)
  } else if (partialCqn.SET && partialCqn.SET.op === 'union') {
    partialCqn.SET.args.forEach(arg => {
      _internalGenerateAliaes(arg, definitions, new Map(aliasMap))
    })
  } else if (partialCqn.xpr) {
    partialCqn.xpr.forEach(arg => {
      _internalGenerateAliaes(arg, definitions, new Map(aliasMap))
    })
  }
}

const generateAliases = (query, model) => {
  if (!query.SELECT) return

  _internalGenerateAliaes(query, model.definitions)
}

module.exports = generateAliases
