const odataParser = require('./odata2cqn')
const cqn2odata = require('./cqn2odata')

const safeNumber = str => {
  const n = Number(str)
  return Number.isSafeInteger(n) || String(n) === str ? n : str
}

const strict = {
  functions: {
    contains: 1,
    startswith: 1,
    endswith: 1,
    tolower: 1,
    toupper: 1,
    length: 1,
    indexof: 1,
    substring: 1,
    trim: 1,
    concat: 1,
    year: 1,
    month: 1,
    day: 1,
    hour: 1,
    minute: 1,
    second: 1,
    time: 1,
    now: 1
  }
}

const odata = (module.exports = {
  parse: {
    url: (url, o) => {
      const options = { safeNumber }
      if (o === 'strict' || (o && o.strict)) options.strict = strict
      return odataParser.parse(url, options)
    },
    cqn: (cqn, kind, model) => {
      const options = { safeNumber }
      return cqn2odata(cqn, kind, options, model)
    }
  },
  to: {
    cqn: (url, o) => odata.parse.url(url, o),
    url: (cqn, kind, model) => odata.parse.cqn(cqn, kind, model)
  }
})
